package foo;

import org.jboss.cache.*;
import org.jboss.cache.config.Configuration;
import org.jboss.cache.lock.IsolationLevel;

public class ConcurrencyTestAtomikos implements Runnable
{
  private static final String KEY = "key"; 
  private Cache cache;
  private Integer number;

  public ConcurrencyTestAtomikos(Cache myCache, Integer myNumber)
  {
    cache = myCache;
    number = myNumber;
  }

  public static void main(String[] args) throws Exception
  {
    Configuration config = new Configuration();
    config.setTransactionManagerLookupClass("foo.AtomikosTransactionManagerLookup");
    config.setIsolationLevel(IsolationLevel.REPEATABLE_READ);
    CacheFactory factory = new DefaultCacheFactory();
    Cache cache = factory.createCache(config);
    cache.create();
    cache.start();

    Node root = cache.getRoot();
    root.put(KEY, new Integer(0));

    ConcurrencyTestAtomikos thread1 = new ConcurrencyTestAtomikos(cache, new Integer(1));
    ConcurrencyTestAtomikos thread2 = new ConcurrencyTestAtomikos(cache, new Integer(2));
    new Thread(thread1, "Thread1").start();
    new Thread(thread2, "Thread2").start();
    //cache.stop();
    //cache.destroy();
    //utm.close();
  }

  @Override
  public void run()
  {
    try
    {
      System.out.println("#" + number + " " + System.nanoTime() + " entering run().");
      javax.transaction.UserTransaction userTransaction = new com.atomikos.icatch.jta.UserTransactionImp();
      userTransaction.begin();
      System.out.println("#" + number + " started transaction.");
      if (number.equals(new Integer(2)))
      {
        try
        {
          System.out.println("#" + number + " waiting.");
          Thread.sleep(2000);
        }
        catch (Throwable t)
        {
          t.printStackTrace();
        }
      }
      Node root = cache.getRoot();
      System.out.println("#" + number + " " + System.nanoTime() + " before put: " + root.get(KEY));
      root.put(KEY, number);
      System.out.println("#" + number + " " + System.nanoTime() + " after put: " + root.get(KEY));
      userTransaction.commit();
      System.out.println("#" + number + " after commit.");
    }
    catch (Throwable t)
    {
      t.printStackTrace();
    }
  }
}
